"""High-level services for MXCP functionality.

This package contains services that orchestrate multiple components
to provide complex functionality like endpoint execution, testing,
evaluation, and drift detection.
"""
